#include "windows.h"
#include <stdio.h>
//#include <conio.h>
#ifdef SGI_GL
#include <SGIGL\gl.h>
#include <SGIGL\glu.h>
#else
#include <gl\gl.h>
#include <gl\glu.h>
#endif

HDC  hdc;
HWND hwnd;
HGLRC hRC;
int  iPixelFormat, iMaxPF,iPFdef, iPT, iC, iZ, iM = 0;

PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL, 						// Format Must Support OpenGL
//		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		16,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};


LONG WINAPI WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    LONG        lRet = 1;
    PAINTSTRUCT ps;

    switch(uMsg) {
    case WM_CREATE:
        break; 

    case WM_DESTROY:
        break; 

    case WM_PAINT: 
        BeginPaint(hWnd, &ps); 
        EndPaint(hWnd, &ps); 
        break; 

    default: 
        lRet = DefWindowProc (hWnd, uMsg, wParam, lParam); 
        break; 
    }

    return lRet; 
} 


/* oglCreateWindow
 * Create a window suitable for OpenGL rendering */
HWND oglCreateWindow(char* title, int x, int y, int width, int height)
{
    WNDCLASS  wc;
    HWND      hWnd;
    HINSTANCE hInstance;

    /* get this modules instance */
    hInstance = GetModuleHandle(NULL);

    /* fill in the window class structure */
    wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC; /* no special styles */
    wc.lpfnWndProc   = (WNDPROC)WindowProc;         /* event handler */
    wc.cbClsExtra    = 0;                           /* no extra class data */
    wc.cbWndExtra    = 0;                           /* no extra window data */
    wc.hInstance     = hInstance;                   /* instance */
    wc.hIcon         = LoadIcon(NULL, IDI_WINLOGO); /* load a default icon */
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW); /* load a default cursor */
    wc.hbrBackground = NULL;                        /* redraw our own bg */
    wc.lpszMenuName  = NULL;                        /* no menu */
    wc.lpszClassName = title;                       /* use a special class */

    /* register the window class */
    if (!RegisterClass(&wc)) {
      MessageBox(NULL, 
		   "RegisterClass() failed:  Cannot register window class,",
		   "Error", MB_OK);
	return NULL;
    }

    /* create a window */
    hWnd = CreateWindow(title,          /* class */
			title,          /* title (caption) */
			WS_CLIPSIBLINGS | WS_CLIPCHILDREN,  /* style */
			x, y, width, height, /* dimensions */
			NULL,		/* no parent */
			NULL,		/* no menu */
			hInstance,	/* instance */
			NULL);		/* don't pass anything to WM_CREATE */

    /* make sure we got a window */
    if (hWnd == NULL) {
	MessageBox(NULL,
		   "CreateWindow() failed:  Cannot create a window.",
		   "Error", MB_OK);
	return NULL;
    }

    return hWnd;
}


static void
dumpvar(char * s, int v)
{
int ii;
glGetIntegerv(v,&ii); printf ("%s%d\n",s,ii);
}

static void
dumpvar2(char * s, int v)
{
int ii;
glGetIntegerv(v,&ii); printf ("%s%dx%d\n",s,ii,ii);
}

/*
 * Print a list of extensions, with word-wrapping.
 */
static void
print_extension_list(const char *ext)
{
   const char *indentString = "    ";
   const int indent = 4;
   int maxl = 79;
   int width, i, j;

   if (!ext || !ext[0])
      return;

   if (iM) maxl = 131;

   width = indent;
   printf(indentString);
   i = j = 0;
   for (;;) {
      if (ext[j] == ' ' || ext[j] == 0) {
         /* found end of an extension name */
         const int len = j - i;
         if (width + len > maxl) {
            /* start a new line */
            printf("\n");
            width = indent;
            printf(indentString);
         }
         /* print the extension name between ext[i] and ext[j] */
         while (i < j) {
            printf("%c", ext[i]);
            i++;
         }
         /* either we're all done, or we'll continue with next extension */
         width += len + 1;
         if (ext[j] == 0) {
            break;
         }
         else {
            i++;
            j++;
            if (ext[j] == 0)
               break;
            printf(", ");
            width += 2;
         }
      }
      j++;
   }
   printf("\n");
}


static void
print_visual_attribs_short_header(void)
{
 printf(" visual accl rgba d st w b g o a g n s s colorbuffer ax dp st accumbuffer\n");
 printf(" id dep type indx b ro n m d g c f p p b  r  g  b  a bf th cl  r  g  b  a\n");
 printf("-------------------------------------------------------------------------\n");
}

static void
print_visual_attribs_short(PIXELFORMATDESCRIPTOR *attribs, int id)
{
char fmt[5], fmt2[5];

int generic_format = attribs->dwFlags & PFD_GENERIC_FORMAT;
int generic_accelerated = attribs->dwFlags & PFD_GENERIC_ACCELERATED;

if (generic_format && !generic_accelerated)
   strcpy(fmt,"soft");
else if (generic_format && generic_accelerated)
   strcpy(fmt,"MCD ");
else if (!generic_format && !generic_accelerated)
   strcpy(fmt,"ICD ");
else
   strcpy(fmt,"unkn");

if (attribs->iPixelType == PFD_TYPE_RGBA) strcpy(fmt2,"RGBA");
else
if (attribs->iPixelType == PFD_TYPE_COLORINDEX) strcpy(fmt2,"INDX");
else
   strcpy(fmt2,"unkn");


   printf("0x%02x %2d %s %s %c  %c %c %c %c %c %c %c %c %c %c",
          id,
          attribs->cColorBits,
          fmt,
          fmt2,
          (attribs->dwFlags & PFD_DOUBLEBUFFER) ? 'y' : '.',
          (attribs->dwFlags & PFD_STEREO) ? 'y' : '.',
          (attribs->dwFlags & PFD_DRAW_TO_WINDOW) ? 'y' : '.',
          (attribs->dwFlags & PFD_DRAW_TO_BITMAP) ? 'y' : '.',
          (attribs->dwFlags & PFD_SUPPORT_GDI) ? 'y' : '.',
          (attribs->dwFlags & PFD_SUPPORT_OPENGL) ? 'y' : '.',
          (attribs->dwFlags & PFD_GENERIC_ACCELERATED) ? 'y' : '.',
          (attribs->dwFlags & PFD_GENERIC_FORMAT) ? 'y' : '.',
          (attribs->dwFlags & PFD_NEED_PALETTE) ? 'y' : '.',
          (attribs->dwFlags & PFD_NEED_SYSTEM_PALETTE) ? 'y' : '.',
          (attribs->dwFlags & PFD_SWAP_LAYER_BUFFERS) ? 'y' : '.'
          );

   printf(" %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d\n",
          attribs->cRedBits, attribs->cGreenBits,
          attribs->cBlueBits, attribs->cAlphaBits,
          attribs->cAuxBuffers,
          attribs->cDepthBits,
          attribs->cStencilBits,
          attribs->cAccumRedBits, attribs->cAccumGreenBits,
          attribs->cAccumBlueBits, attribs->cAccumAlphaBits
          );
}

static void
print_visual_attribs_long_header(void)
{
 printf("Vis  Vis  Accl Color DB ste drw drw gdi ogl gen gen ned sys swp  r   g   b   a  aux dep ste  accum buffers\n");
 printf(" ID Depth Type Type     reo win bmp sup sup acc fmt pal pal buf sz  sz  sz  sz  buf th  ncl  r   g   b   a\n");
 printf("----------------------------------------------------------------------------------------------------------\n");
}

static void
print_visual_attribs_long(PIXELFORMATDESCRIPTOR *attribs, int id)
{
char fmt[5], fmt2[6];

int generic_format = attribs->dwFlags & PFD_GENERIC_FORMAT;
int generic_accelerated = attribs->dwFlags & PFD_GENERIC_ACCELERATED;

if (generic_format && !generic_accelerated)
   strcpy(fmt,"soft");
else if (generic_format && generic_accelerated)
   strcpy(fmt,"MCD ");
else if (!generic_format && !generic_accelerated)
   strcpy(fmt,"ICD ");
else
   strcpy(fmt,"unkn");

if (attribs->iPixelType == PFD_TYPE_RGBA) strcpy(fmt2,"RGBA ");
else
if (attribs->iPixelType == PFD_TYPE_COLORINDEX) strcpy(fmt2,"INDEX");
else
   strcpy(fmt2,"unkn ");

   printf("0x%02x %2d   %s %s %c   %c   %c   %c   %c   %c   %c   %c   %c   %c   %c",
          id,
          attribs->cColorBits,
          fmt,
          fmt2,
          (attribs->dwFlags & PFD_DOUBLEBUFFER) ? 'y' : '.',
          (attribs->dwFlags & PFD_STEREO) ? 'y' : '.',
          (attribs->dwFlags & PFD_DRAW_TO_WINDOW) ? 'y' : '.',
          (attribs->dwFlags & PFD_DRAW_TO_BITMAP) ? 'y' : '.',
          (attribs->dwFlags & PFD_SUPPORT_GDI) ? 'y' : '.',
          (attribs->dwFlags & PFD_SUPPORT_OPENGL) ? 'y' : '.',
          (attribs->dwFlags & PFD_GENERIC_ACCELERATED) ? 'y' : '.',
          (attribs->dwFlags & PFD_GENERIC_FORMAT) ? 'y' : '.',
          (attribs->dwFlags & PFD_NEED_PALETTE) ? 'y' : '.',
          (attribs->dwFlags & PFD_NEED_SYSTEM_PALETTE) ? 'y' : '.',
          (attribs->dwFlags & PFD_SWAP_LAYER_BUFFERS) ? 'y' : '.'
          );

   printf("  %2d  %2d  %2d  %2d  %2d  %2d  %2d  %2d  %2d  %2d  %2d\n",
          attribs->cRedBits, attribs->cGreenBits,
          attribs->cBlueBits, attribs->cAlphaBits,
          attribs->cAuxBuffers,
          attribs->cDepthBits,
          attribs->cStencilBits,
          attribs->cAccumRedBits, attribs->cAccumGreenBits,
          attribs->cAccumBlueBits, attribs->cAccumAlphaBits
          );
}

static void
enum_pf(void)
{

iPixelFormat = 1;
iMaxPF = DescribePixelFormat(hdc, iPixelFormat, 
        sizeof(PIXELFORMATDESCRIPTOR), &pfd);
iPFdef = GetPixelFormat(hdc);

if (!iM)
{
print_visual_attribs_short_header();
for (iPixelFormat;iPixelFormat <= iMaxPF; iPixelFormat++)
{
DescribePixelFormat(hdc,iPixelFormat,sizeof(PIXELFORMATDESCRIPTOR), &pfd);
print_visual_attribs_short(&pfd,iPixelFormat);
}
} else {
print_visual_attribs_long_header();
for (iPixelFormat;iPixelFormat <= iMaxPF; iPixelFormat++)
{
DescribePixelFormat(hdc,iPixelFormat,sizeof(PIXELFORMATDESCRIPTOR), &pfd);
print_visual_attribs_long(&pfd,iPixelFormat);
}
}                     
}

static void
usage(void)
{
   printf("Usage: glxinfo [-v] [-t] [-h]\n");
//   printf("\t-v: Print visuals info in verbose form.\n");
   printf("\t-t: Print verbose table.\n");
//   printf("\t-display <dname>: Print GLX visuals on specified server.\n");
   printf("\t-h: This information.\n");
//   printf("\t-i: Force an indirect rendering context.\n");
//   printf("\t-b: Find the 'best' visual and print it's number.\n");
//   printf("\t-l: Print interesting OpenGL limits.\n");
}

int
main(int argc, char *argv[])
{
   int i;

   for (i = 1; i < argc; i++) {
      if (strcmp(argv[i], "-v") == 0) {
         iM = 1;
      }
      else if (strcmp(argv[i], "-h") == 0) {
         usage();
         return 0;
      }
      else {
         printf("Unknown option `%s'\n", argv[i]);
         usage();
         return 0;
      }
   }

////////////////////////////////////////

if (hwnd = oglCreateWindow("OpenGL", 0, 0, 200, 200))
{
if (hdc = GetDC(hwnd))
{
printf("display resolution: %dx%d, %d bpp, %d plane[s], %d hz\n",
GetDeviceCaps(hdc,HORZRES),
GetDeviceCaps(hdc,VERTRES),GetDeviceCaps(hdc,BITSPIXEL),
GetDeviceCaps(hdc,PLANES),GetDeviceCaps(hdc,VREFRESH));
printf("display driver version: 0x%04x\n",GetDeviceCaps(hdc,DRIVERVERSION));
printf("display size (in mm): %d x %d\n",GetDeviceCaps(hdc,HORZSIZE),
GetDeviceCaps(hdc,VERTSIZE));
printf("display logical pixels: %d x %d\n",GetDeviceCaps(hdc,LOGPIXELSX),
GetDeviceCaps(hdc,LOGPIXELSY));
printf("display aspect: x %d y %d xy %d\n",GetDeviceCaps(hdc,ASPECTX),
GetDeviceCaps(hdc,ASPECTY),GetDeviceCaps(hdc,ASPECTXY));
printf("display palette size: %d\n",GetDeviceCaps(hdc,SIZEPALETTE));
printf("display reserved colors: %d\n",GetDeviceCaps(hdc,NUMRESERVED));
printf("display color resolution: %d\n",GetDeviceCaps(hdc,COLORRES));
printf("display raster caps: 0x%04x\n",GetDeviceCaps(hdc,RASTERCAPS));
printf("display curve caps: 0x%04x\n",GetDeviceCaps(hdc,CURVECAPS));
printf("display line caps: 0x%04x\n",GetDeviceCaps(hdc,LINECAPS));
printf("display polygonal caps: 0x%04x\n",GetDeviceCaps(hdc,POLYGONALCAPS));
printf("display text caps: 0x%04x\n",GetDeviceCaps(hdc,TEXTCAPS));
printf("display colormangement caps: 0x%04x\n",GetDeviceCaps(hdc,COLORMGMTCAPS));
printf("display shadeblend caps: 0x%04x\n",GetDeviceCaps(hdc,SHADEBLENDCAPS));
pfd.cColorBits = pfd.cDepthBits = GetDeviceCaps(hdc,BITSPIXEL);
iPFdef = ChoosePixelFormat(hdc, &pfd); 
if (!iPFdef) printf("glx_warn: ChoosePixelFormat() == 0\n");
if (SetPixelFormat(hdc,iPFdef, &pfd) == FALSE) printf("glx_warn: SetPixelFormat() == FALSE\n");
hRC = wglCreateContext(hdc);
if (hRC == NULL) printf("glx_warn: wglCreateContext() == 0\n");
if (wglMakeCurrent(hdc,hRC) == FALSE) printf("glx_warn: wglMakeCurrent() == 0\n");
printf("\nOpenGL vendor string: %s\n",glGetString(GL_VENDOR));
printf("OpenGL renderer string: %s\n",glGetString(GL_RENDERER));
printf("OpenGL version string: %s\n",glGetString(GL_VERSION));
printf("OpenGL extensions:\n");
print_extension_list((char*)glGetString(GL_EXTENSIONS));
printf("GLU version string: %s\n",gluGetString(GLU_VERSION));
printf("GLU extensions:\n");
print_extension_list((char*)gluGetString(GLU_EXTENSIONS));
//getch(); 
printf("\n");
dumpvar2("max texture size: ",GL_MAX_TEXTURE_SIZE);
dumpvar2("max viewport dims: ",GL_MAX_VIEWPORT_DIMS);
dumpvar("max modelview stack depth: ",GL_MAX_MODELVIEW_STACK_DEPTH);
dumpvar("max projection stack depth: ",GL_MAX_PROJECTION_STACK_DEPTH);
dumpvar("max texture stack depth: ",GL_MAX_TEXTURE_STACK_DEPTH);
dumpvar("max attrib stack depth: ",GL_MAX_ATTRIB_STACK_DEPTH);
dumpvar("max client attrib stack depth: ",GL_MAX_CLIENT_ATTRIB_STACK_DEPTH);
dumpvar("max name stack depth: ",GL_MAX_NAME_STACK_DEPTH);
dumpvar("max list nesting: ",GL_MAX_LIST_NESTING);
dumpvar("max eval order: ",GL_MAX_EVAL_ORDER);
dumpvar("max pixel map table: ",GL_MAX_PIXEL_MAP_TABLE);
dumpvar("max lights: ",GL_MAX_LIGHTS);
dumpvar("max clip planes: ",GL_MAX_CLIP_PLANES);
dumpvar("aux buffers: ",GL_AUX_BUFFERS);
dumpvar("subpixel bits: ",GL_SUBPIXEL_BITS);
printf("\nglx_info: ChoosePixelFormat() == 0x%02x\n\n",iPFdef);
enum_pf();
if (wglMakeCurrent(NULL,NULL) == FALSE) printf("glx_warn: wglMakeCurrent(NULL) == FALSE\n");
if (wglDeleteContext(hRC) == FALSE) printf("glx_warn: wglDeleteContext() == FALSE\n");
if (ReleaseDC(hwnd,hdc) == 0) printf("glx_warn: ReleaseDC() == 0\n");
}
else
{ printf("glx_err: GetDC() == 0\n"); return 1; }
}
else
{ printf("glx_err: oglCreateWindow() == 0\n"); return 1; }
//getch(); 
////////////////////////////////////////
   return 0;
}

